@@ -22,10 +22,11 @@ STATUS_NAMES = ["status", "s"]
 HOURS_NAMES =  ["hours", "h"]
 LIST_NAMES =   ["list", "ls"]
 DAY_NAME =      "day"
+VIEW_NAME =     "view"
 EXPORT_NAME =   "export"
 EDIT_NAMES =   ["edit", "e"]
 REMOVE_NAMES = ["remove", "rm"]
-RECESS_NAME =   "recess"
+RECESS_NAMES =   ["free-days", "recess"]
 
 CONFIG_NAME =   "config"
 REHASH_NAME =   "rehash"
@@ -39,12 +40,12 @@ class Mode:
     def __init__(
         self,
         names: Union[str, List[str]],
-        help: Optional[str] = None,
+        help_text: Optional[str] = None,
         description: Optional[str] = None,
     ) -> None:
         self.names = [names] if isinstance(names, str) else names
-        self.help = help
-        self.description = description or help
+        self.help = help_text
+        self.description = description or help_text
         self.parents: List[argparse.ArgumentParser] = []
 
     def add_as_parser(self, subparsers) -> argparse.ArgumentParser:
@@ -62,6 +63,7 @@ class Mode:
         )
 
     def create_fish_completion(self, all_modes: List[str]) -> str:
+        """ Convert given arguments to fish completion. """
         for field in ["help", "description"]:
             if self.__dict__[field]:
                 self.__dict__[field] = self.__dict__[field].replace('"', '\\"')
@@ -76,24 +78,26 @@ class Mode:
 
 
 MODES: Dict[str, Mode] = {
-    START_NAME: Mode(START_NAME, help="Start the protocol"),
-    STOP_NAME: Mode(STOP_NAME, help="Stop the protocol"),
-    ADD_NAME: Mode(ADD_NAME, help="Add a protocol entry"),
+    START_NAME: Mode(START_NAME, help_text="Start the protocol"),
+    STOP_NAME: Mode(STOP_NAME, help_text="Stop the protocol"),
+    ADD_NAME: Mode(ADD_NAME, help_text="Add a protocol entry"),
     SWITCH_NAMES[0]: Mode(
         SWITCH_NAMES,
-        help="Short-hand for stop & start",
+        help_text="Short-hand for stop & start",
         description=(
             "A short-hand for stop and start. Flags are passed to stop, meaning they "
             "refer to the stopped run."
         ),
     ),
-    CANCEL_NAME: Mode(CANCEL_NAME, help="Cancel the current run"),
-    RESUME_NAME: Mode(RESUME_NAME, help='Resume the last run today (undo "work stop")'),
-    STATUS_NAMES[0]: Mode(STATUS_NAMES, help="Print the current status"),
-    HOURS_NAMES[0]: Mode(HOURS_NAMES, help="Calculate hours worked"),
+    CANCEL_NAME: Mode(CANCEL_NAME, help_text="Cancel the current run"),
+    RESUME_NAME: Mode(
+        RESUME_NAME, help_text='Resume the last run today (undo "work stop")'
+    ),
+    STATUS_NAMES[0]: Mode(STATUS_NAMES, help_text="Print the current status"),
+    HOURS_NAMES[0]: Mode(HOURS_NAMES, help_text="Calculate hours worked"),
     LIST_NAMES[0]: Mode(
         LIST_NAMES,
-        help="List protocol records",
+        help_text="List protocol records",
         description=(
             "List protocol records (by default the current day)."
             " For other ranges use the optional arguments."
@@ -101,31 +105,41 @@ MODES: Dict[str, Mode] = {
     ),
     DAY_NAME: Mode(
         DAY_NAME,
-        help="List the current day with all details.",
+        help_text="List the current day with all details.",
         description=(
             "List the current day with all details. Shorthand for "
             '"list --include-active --with-breaks --list-empty"'
         ),
     ),
-    EXPORT_NAME: Mode(EXPORT_NAME, help="Export records as CSV."),
+    VIEW_NAME: Mode(
+        VIEW_NAME,
+        help_text="Views on protocol records",
+        description=(
+            "View protocol records (by default the current day). "
+            "Similar to list, but summarizes by aspects other than date."
+        ),
+    ),
+    EXPORT_NAME: Mode(EXPORT_NAME, help_text="Export records as CSV."),
     EDIT_NAMES[0]: Mode(
         EDIT_NAMES,
-        help="Edit protocol records",
+        help_text="Edit protocol records",
         description="Edit protocol records (by default from the current day)",
     ),
     REMOVE_NAMES[0]: Mode(
         REMOVE_NAMES,
-        help="Remove records from the protocol",
+        help_text="Remove records from the protocol",
         description="Remove records from the protocol (by default from the current day)",
     ),
-    RECESS_NAME: Mode(
-        RECESS_NAME,
-        help="Manage free days (vacation, holidays, ...)",
+    RECESS_NAMES[0]: Mode(
+        RECESS_NAMES,
+        help_text="Manage free days (vacation, holidays, ...)",
         description="Manage free days (vacation, holidays, ...). Default mode: --list",
     ),
-    CONFIG_NAME: Mode(CONFIG_NAME, help="Check and interact with the configuration"),
+    CONFIG_NAME: Mode(
+        CONFIG_NAME, help_text="Check and interact with the configuration"
+    ),
     REHASH_NAME: Mode(
-        REHASH_NAME, help="Recompute verification checksum after manual edits"
+        REHASH_NAME, help_text="Recompute verification checksum after manual edits"
     ),
 }
 
@@ -142,8 +156,9 @@ class Arguments:
             epilog="To find out more, check the help page of individual modes.",
         )
         # fmt: off
-        parser.add_argument("-H", "--help-verbose", action="store_true", help="A longer help output, similar to a man page.")
-        parser.add_argument("-v", "--version", action="version", version=f"%(prog)s {version}")
+        parser.add_argument("-H", "--help-verbose", action="store_true",
+            help="A longer help output, similar to a man page.")
+        parser.add_argument("-V", "--version", action="version", version=f"%(prog)s {version}")
         parser.add_argument("-d", "--debug", action="store_true", help="Use a debug directory")
         parser.add_argument("-y", "--dry-run", action="store_true", help="Only print output")
         # fmt: on
@@ -151,18 +166,27 @@ class Arguments:
         modes = parser.add_subparsers(title="modes", dest="mode")
 
         # Shared help texts
-        date_help_text = 'Either a date (such as "12.", "1.1." or "5.09.19"), or any prefix of "today" or "yesterday".'
+        date_help_text = (
+            'Either a date (such as "12.", "1.1." or "5.09.19"), or any prefix of '
+            '"today" or "yesterday".'
+        )
         # TODO Consider if / how the keyword "again" should be documented
-        time_help_text = 'Either a time (such as "1:20" or "12:1") or "now" for the current time (rounded).'
+        time_help_text = (
+            'Either a time (such as "1:20" or "12:1") or "now" for the current time '
+            "(rounded)."
+        )
 
-        # Parent parser for optional entry fields
+        ## Parent parser for optional entry fields ##
 
         category_message_parent = argparse.ArgumentParser(add_help=False)
         category_message_parent.add_argument(
             "-c",
             "--category",
             metavar="C",
-            help="Categorize the entry. Anything is allowed, but this will be used for summarization.",
+            help=(
+                "Categorize the entry. Anything is allowed, but this will be used for "
+                "summarization."
+            ),
         )
         category_message_parent.add_argument(
             "-m", "--message", metavar="M", help="Free-text description of the entry."
@@ -172,6 +196,76 @@ class Arguments:
         MODES[ADD_NAME].parents.append(category_message_parent)
         MODES[SWITCH_NAMES[0]].parents.append(category_message_parent)
 
+        ## Parent parsers for dates ##
+
+        single_date_sel_parent = argparse.ArgumentParser(add_help=False)
+        single_date_sel_modes = single_date_sel_parent.add_mutually_exclusive_group()
+        multi_date_sel_parent = argparse.ArgumentParser(add_help=False)
+        multi_date_sel_modes = multi_date_sel_parent.add_mutually_exclusive_group()
+
+        # The alternative (two disjunct parents) would not allow the exclusivity guarantee
+        for date_sel_modes in [single_date_sel_modes, multi_date_sel_modes]:
+            date_sel_modes.add_argument(
+                "-d", "--date", metavar="DATE", help=f"Any DATE – {date_help_text}"
+            )
+            date_sel_modes.add_argument(
+                "-1",
+                "--yesterday",
+                action="store_const",
+                dest="date",
+                const="yesterday",
+                help="Short-hand for --date yesterday.",
+            )
+            date_sel_modes.add_argument(
+                "-D",
+                "--day",
+                metavar="DAY",
+                help=(
+                    'A weekday or any matching prefix, e.g. "Mon", "su" or "wednesday". '
+                    "Always selects from the past seven days."
+                ),
+            )
+        multi_date_sel_modes.add_argument(
+            "-p",
+            "--period",
+            metavar="DATE",
+            nargs=2,
+            help="A specified period, defined by two DATEs.",
+        )
+        multi_date_sel_modes.add_argument(
+            "-s",
+            "--since",
+            metavar="DATE",
+            help="The period between DATE and today.",
+        )
+        multi_date_sel_modes.add_argument(
+            "-w",
+            "--week",
+            metavar="W",
+            type=int,
+            nargs="?",
+            const=-1,
+            help="The current week (no argument) or week no. W of this year.",
+        )
+        multi_date_sel_modes.add_argument(
+            "-m",
+            "--month",
+            metavar="DATE",
+            nargs="?",
+            const="today",
+            help="The current month (no argument) or the month containing DATE.",
+        )
+
+        # Single date selection
+        MODES[ADD_NAME].parents.append(single_date_sel_parent)
+        MODES[EDIT_NAMES[0]].parents.append(single_date_sel_parent)
+        MODES[REMOVE_NAMES[0]].parents.append(single_date_sel_parent)
+
+        # Multi date selection
+        MODES[LIST_NAMES[0]].parents.append(multi_date_sel_parent)
+        MODES[VIEW_NAME].parents.append(multi_date_sel_parent)
+        MODES[EXPORT_NAME].parents.append(multi_date_sel_parent)
+
         # SINGLE TIME modes
 
         start_mode = MODES[START_NAME].add_as_parser(modes)
@@ -193,7 +287,6 @@ class Arguments:
 
         add_mode = MODES[ADD_NAME].add_as_parser(modes)
         add_mode.set_defaults(func=program.add)
-        add_mode.add_argument("-d", "--date", metavar="DATE", help=date_help_text)
         add_mode.add_argument(
             "time_from", metavar="TIME", help="Start time. " + time_help_text
         )
@@ -293,51 +386,12 @@ class Arguments:
             "--start",
             metavar="H:M",
             dest="h_start",
-            help="Override the start time assumed by hours. May only be used when no run is active.",
+            help=(
+                "Override the start time assumed by hours. May only be used when no "
+                "run is active."
+            ),
         )
 
-        # Parent parser for date ranges
-        date_selection_parent = argparse.ArgumentParser(add_help=False)
-        date_selection_modes = date_selection_parent.add_mutually_exclusive_group()
-        date_selection_modes.add_argument(
-            "-d", "--date", metavar="DATE", help=f"Any DATE – {date_help_text}"
-        )
-        date_selection_modes.add_argument(
-            "-1",
-            "--yesterday",
-            action="store_const",
-            dest="date",
-            const="yesterday",
-            help="Short-hand for --date yesterday.",
-        )
-        date_selection_modes.add_argument(
-            "-D",
-            "--day",
-            metavar="DAY",
-            help='A weekday (full or abbreviated), such as "Mon", "su" or "wednesday".',
-        )
-        date_selection_modes.add_argument(
-            "-p",
-            "--period",
-            metavar="DATE",
-            nargs=2,
-            help="A specified period, defined by two DATEs.",
-        )
-        date_selection_modes.add_argument(
-            "-w",
-            "--week",
-            type=int,
-            nargs="?",
-            const=-1,
-            help="The specified week (default: current week). Expects a week number.",
-        )
-        date_selection_modes.add_argument(
-            "-m", "--month", action="store_true", help="The current month."
-        )
-
-        MODES[LIST_NAMES[0]].parents.append(date_selection_parent)
-        MODES[EXPORT_NAME].parents.append(date_selection_parent)
-
         # PROTOCOL interaction modes
 
         list_mode = MODES[LIST_NAMES[0]].add_as_parser(modes)
@@ -364,6 +418,34 @@ class Arguments:
         day_mode = MODES[DAY_NAME].add_as_parser(modes)
         day_mode.set_defaults(func=program.day)
 
+        view_mode = MODES[VIEW_NAME].add_as_parser(modes)
+        view_mode.set_defaults(func=program.view)
+        view_mode.add_argument(
+            "mode",
+            help=(
+                "View mode. by-category summarizes based on category instead of date. "
+                "balance shows balance development over time."
+            ),
+            choices=["by-category", "balance"],
+        )
+
+        for listing_mode in [list_mode, view_mode]:
+            listing_mode.add_argument(
+                "--filter-category",
+                "--Fc",
+                metavar="PATTERN",
+                help=(
+                    "Filter: Only include records with matching category. Supports "
+                    "glob patterns."
+                ),
+            )
+            listing_mode.add_argument(
+                "--filter-message",
+                "--Fm",
+                metavar="PATTERN",
+                help="Filter: Like --filter-category, but for the message.",
+            )
+
         export_mode = MODES[EXPORT_NAME].add_as_parser(modes)
         export_mode.set_defaults(func=program.export)
 
@@ -373,14 +455,9 @@ class Arguments:
         remove_mode = MODES[REMOVE_NAMES[0]].add_as_parser(modes)
         remove_mode.set_defaults(func=program.remove)
 
-        for manipulation_mode in [edit_mode, remove_mode]:
-            manipulation_mode.add_argument(
-                "-d", "--date", metavar="DATE", help=date_help_text
-            )
-
         # RECESS management
 
-        recess_mode = MODES[RECESS_NAME].add_as_parser(modes)
+        recess_mode = MODES[RECESS_NAMES[0]].add_as_parser(modes)
         recess_mode.set_defaults(func=program.recess)
         recess_mode_modes = recess_mode.add_mutually_exclusive_group()
         recess_mode_modes.add_argument(
@@ -399,8 +476,9 @@ class Arguments:
             "--add-reduced-day",
             nargs=2,
             metavar=("DATE", "HOURS"),
-            help="Add a reduced hour day on DATE (see DATE) with HOURS being a value in ({}, {}).".format(
-                *consts.ALLOWED_WORK_HOURS
+            help=(
+                "Add a reduced hour day on DATE (see DATE) with HOURS being a value in "
+                "({}, {}).".format(*consts.ALLOWED_WORK_HOURS)
             ),
         )
         recess_mode_modes.add_argument(
