@@ -3,14 +3,15 @@
 """ Shared constants """
 
 import os
+import pathlib
 from typing import List, Tuple
 
 PROTOCOL_VERSION = 3
 
 # Directories
-PREFIX = os.path.expanduser("~/.local/share")
-DIRECTORY = os.path.join(PREFIX, "work")
-DIRECTORY_DEBUG = os.path.join(PREFIX, "debug", "work")
+PARENT_DIR: pathlib.Path = pathlib.Path("~", ".local", "share").expanduser()
+DIRECTORY: pathlib.Path = PARENT_DIR.joinpath("work")
+DIRECTORY_DEBUG: pathlib.Path = PARENT_DIR.joinpath("debug", "work")
 
 # Paths – work directory
 PROTOCOL_DIRECTORY_NAME = "records"
