@@ -2,11 +2,14 @@
 
 """ Protocol wrappers """
 
+import atexit
 import csv
 import datetime as dt
 import os
 import pathlib
 import re
+import shutil
+import tempfile
 from typing import Iterable, List, Optional
 
 from work_components import util
@@ -18,11 +21,12 @@ from work_components.consts import (
     TIME_FORMAT,
     YEAR_DIR_PATTERN,
 )
+from work_components.timestamps import date_equals
 
 ### Record container classes ###
 
 
-class Record(object):
+class Record:
     """ A protocol record. Invariant: start <= end """
 
     def __init__(
@@ -47,6 +51,13 @@ class Record(object):
     def end(self) -> dt.datetime:
         return self._end
 
+    @property
+    def date(self) -> dt.date:
+        """ This record's date. Raises if start date != end date. """
+        if self.start.date() != self.end.date():
+            raise ValueError("Record end date differs from start date!")
+        return self.start.date()
+
     @property
     def category(self) -> str:
         return self._category
@@ -76,7 +87,8 @@ class Record(object):
     def merge(self, other):
         # type: (Record) -> Record
         """
-        Try to merge two records. Can only merge touching or overlapping records that are otherwise identical.
+        Try to merge two records. Can only merge touching or overlapping records that
+        are otherwise identical.
 
         Raises if the merge is unsuccessful.
         """
@@ -182,7 +194,7 @@ class Record(object):
 ### Protocol container classes ###
 
 
-class ProtocolMeta(object):
+class ProtocolMeta:
     def __init__(self):
         self._path = "NOT_DEFINED"
         raise NotImplementedError()
@@ -252,7 +264,7 @@ class ProtocolDay(ProtocolMeta):
     def entries(self) -> List[Record]:
         """ The entries on this day. """
 
-        if not os.path.lexists(self.path):
+        if not self.path.exists():
             return []
 
         entries: List[Record] = []
@@ -266,7 +278,7 @@ class ProtocolDay(ProtocolMeta):
                     record = Record.from_protocol_row(row)
                     self._raise_if_invalid_for_me(record)
                 except Record.CorruptedProtocolRowError as err:
-                    raise IOError(f"{err}\n  In: {self.path}")
+                    raise IOError(f"{err}\n  In: {self.path}") from err
                 entries.append(record)
 
         if 0 == len(entries):
@@ -276,9 +288,9 @@ class ProtocolDay(ProtocolMeta):
 
     def _raise_if_invalid_for_me(self, record: Record):
         """ Check if the record matches this ProtocolDay and raise if not. """
-        if record.start.date() != self.date:
+        if not date_equals(record.start, self.date):
             raise Record.CorruptedProtocolRowError("date does not match file path")
-        if record.start.date() != record.end.date():
+        if not date_equals(record.start, record.end):
             raise Record.CorruptedProtocolRowError("start and end date differ")
 
     def add(self, record: Record, force: bool = False) -> None:
@@ -297,8 +309,14 @@ class ProtocolDay(ProtocolMeta):
         entries.append(record)
         try:
             entries_processed: List[Record] = sort_and_merge(entries)
-        except ValueError as ve:
-            raise ValueError(f"{str(ve)}\nIn file: {self.path}")
+        except OverlapError as ovl_err:
+            ovl_err.message = (
+                "Overlapping entries detected – please fix manually. Offending records:\n"
+                f"  {ovl_err.left.strf(TIME_FORMAT)}\n"
+                f"  {ovl_err.right.strf(TIME_FORMAT)}\n"
+                f"In file: {self.path}"
+            )
+            raise ovl_err
 
         self._write_out(records=entries_processed)
 
@@ -341,7 +359,8 @@ class ProtocolDay(ProtocolMeta):
                 processed_entries.extend([entry_left, entry_right])
                 print(
                     f"Existing record split up: {entry_str}\n"
-                    f"  ➜  {entry_left.strftime(TIME_FORMAT)}  &  {entry_right.strftime(TIME_FORMAT)}"
+                    f"  ➜  {entry_left.strftime(TIME_FORMAT)}"
+                    f"  &  {entry_right.strftime(TIME_FORMAT)}"
                 )
                 continue
 
@@ -383,7 +402,7 @@ class ProtocolDay(ProtocolMeta):
 
         # If no entries remain, remove the empty file on disk.
         if 0 == len(entries):
-            os.remove(self.path)
+            self.path.unlink()
             return
 
         self._write_out(records=entries)
@@ -408,6 +427,32 @@ class ProtocolDay(ProtocolMeta):
                 protocol_writer.writerow(row)
 
 
+class ShadowProtocolDay(ProtocolDay):
+    """ A `ProtocolDay` with tempfile storage for dry runs or similar. """
+
+    def __init__(self, date: dt.date):
+        root: pathlib.Path = pathlib.Path(tempfile.mkdtemp())
+        super().__init__(date=date, root_directory=root)
+        atexit.register(lambda: shutil.rmtree(root))
+        atexit.register(self.path.unlink)
+
+    @staticmethod
+    def copy(other):
+        # type: (ProtocolDay) -> ShadowProtocolDay
+        """ Copy `other`s entries to shadow it. """
+        shadow = ShadowProtocolDay(other.date)
+        for entry in other.entries:
+            shadow.add(record=entry)
+        return shadow
+
+    def overwrite(self, other: ProtocolDay):
+        """ Replace `other`s state by own state. """
+        for others_entry in other.entries:
+            other.remove(record=others_entry)
+        for my_entry in self.entries:
+            other.add(record=my_entry)
+
+
 class ProtocolRange(ProtocolMeta):
     """
     Container for an arbitrary time range in the protocol.
@@ -529,8 +574,27 @@ class Protocol(ProtocolMeta):
             yield child
 
 
+class OverlapError(ValueError):
+    """ Entries overlap when they shouldn't. """
+
+    def __init__(self, left: Record, right: Record) -> None:
+        super().__init__()
+        self.left = left
+        self.right = right
+        self.message = f"Overlapping entries: {left} / {right}"
+
+    def __str__(self) -> str:
+        return self.message
+
+        # (
+        #         "Overlapping entries detected – please fix manually. "
+        #         f"Offending records:\n  {last_record}\n  {next_record}"
+        #     )
+
+
 def sort_and_merge(entries: List[Record], output: bool = True) -> List[Record]:
-    """ Sort the given entries based on their start time and merge touching and mergeable entries. """
+    """Sort the given entries based on their start time and merge touching and
+    mergeable entries."""
 
     # Before entries are added, the DAO checks and prevents overlaps. Therefore, we can
     # assume that entries should be either apart or "touch" and raise otherwise.
@@ -555,10 +619,7 @@ def sort_and_merge(entries: List[Record], output: bool = True) -> List[Record]:
 
         # Error case (overlapping entries): Should only occur after manual edits
         if next_record.start < last_record.end:
-            raise ValueError(
-                "Overlapping entries detected – please fix manually. "
-                f"Offending records:\n  {last_record}\n  {next_record}"
-            )
+            raise OverlapError(last_record, next_record)
 
         try:
             merged_record: Record = last_record.merge(next_record)
