@@ -25,12 +25,12 @@ from work_components.consts import RC_FILE_PATH
 TIME_FORMAT = "%H:%M"
 
 
-class RCError(IOError):
+class RCError(OSError):
     def __init__(self, msg):
         super().__init__("RC file erroneous! {}".format(msg))
 
 
-class RC(object):
+class RC:
 
     expected_hours_k = "expected_hours"
 
