@@ -199,8 +199,8 @@ class RecessFile:
 class RecessDao:
     """ Interface to manage recess. Handles `RecessFile`s internally. """
 
-    def __init__(self, work_directory: str) -> None:
-        self.directory: Path = Path(work_directory, RECESS_DIRECTORY_NAME)
+    def __init__(self, work_directory: Path) -> None:
+        self.directory: Path = work_directory.joinpath(RECESS_DIRECTORY_NAME)
         self.recess_files: Dict[int, RecessFile] = {}
 
     def _get(self, year: int) -> RecessFile:
