@@ -26,7 +26,8 @@ from work_components.container import (
 )
 
 
-class WorkDao(object):
+class WorkDao:
+    """ DAO for the work directory and its children. """
 
     # work_directory        self.work_directory
     # |- info.winf          self.info_file
@@ -39,10 +40,12 @@ class WorkDao(object):
     #    |- ...             –
     #
     # (see work_components/consts.py)
-    def __init__(self, work_directory: str):
-        self.work_directory: pathlib.Path = pathlib.Path(work_directory)
+    def __init__(self, work_directory: pathlib.Path):
+        self.work_directory: pathlib.Path = work_directory
         self.info_file: pathlib.Path = self.work_directory.joinpath(INFO_FILE_NAME)
-        self.run_file: pathlib.Path = self.work_directory.joinpath(RUN_FILE_NAME)
+        self.run_file: WorkDao.RunFileDao = WorkDao.RunFileDao(
+            self.work_directory.joinpath(RUN_FILE_NAME)
+        )
         self.records_directory: pathlib.Path = self.work_directory.joinpath(
             PROTOCOL_DIRECTORY_NAME
         )
@@ -60,8 +63,7 @@ class WorkDao(object):
                 f'File "{self.run_file.resolve()}" exists already! Could not create.'
             )
 
-        with self.run_file.open(mode="w") as time_file:
-            time_file.write(start_time.strftime(DATETIME_FORMAT))
+        self.run_file.write(start_time)
 
     def stop_run(self, end_time: dt.datetime, category: str, message: str) -> None:
         """ Stop a run at the given time. Raises on invalid operation. """
@@ -119,7 +121,7 @@ class WorkDao(object):
 
     def run_active(self) -> bool:
         """ Convenience function to check if a run is active. """
-        return self.get_start_time() != None
+        return self.get_start_time() is not None
 
     def get_start_time(self) -> Optional[dt.datetime]:
         """
@@ -131,16 +133,7 @@ class WorkDao(object):
         if not self.run_file.exists():
             return None
 
-        content = None
-        with self.run_file.open(mode="r") as run_file:
-            content = run_file.read()
-
-        try:
-            return dt.datetime.strptime(content, DATETIME_FORMAT)
-        except ValueError:
-            raise IOError(
-                f"Invalid run file! Please delete or fix. (Path: {self.run_file.resolve()})"
-            )
+        return self.run_file.read()
 
     def has_entry(self, start_time: dt.datetime, end_time: dt.datetime = None) -> bool:
         """ Check whether an existing entry overlaps the given time. """
@@ -164,7 +157,8 @@ class WorkDao(object):
         # There may not be any entry on any day overlapping any part of the given slot
         for protocol_day in protocol_days:
             for record in protocol_day.entries:
-                # "Touching" of entries (so start_time == existing_end or end_time == existing_start) is allowed
+                # "Touching" of entries (so start_time == existing_end or
+                # end_time == existing_start) is allowed
                 if record.overlaps(Record(start=start_time, end=end_time)):
                     return True
 
@@ -173,7 +167,7 @@ class WorkDao(object):
     def get_entries(
         self,
         date: Optional[dt.date] = None,
-        range: Optional[Tuple[dt.date, dt.date]] = None,
+        date_range: Optional[Tuple[dt.date, dt.date]] = None,
     ) -> List[Record]:
         """
         Load stored entries (excluding a possible active run).
@@ -189,14 +183,14 @@ class WorkDao(object):
         entries_container: Optional[ProtocolMeta] = None
 
         # Nothing specified: Load all
-        if date is None and range is None:
+        if date is None and date_range is None:
             entries_container = self._load_protocol()
         # Date specified
         elif date is not None:
             entries_container = self._load_protocol_date(date=date)
         # Range specified
-        elif range is not None:
-            entries_container = self._load_protocol_range(*range)
+        elif date_range is not None:
+            entries_container = self._load_protocol_range(*date_range)
         # Both specified: Invalid arguments
         else:
             raise ValueError("May either specifiy a date or a range, not both.")
@@ -233,7 +227,8 @@ class WorkDao(object):
         # If just one exists, something has gone wrong.
         if protocols_exist ^ protocol_info_exists:
             raise IOError(
-                "A required file is missing! (Either the protocol or the protocol info file does not exist.)"
+                "A required file is missing! (Either the protocol or the protocol "
+                "info file does not exist.)"
             )
 
         ifile_protocol_version: int
@@ -251,7 +246,8 @@ class WorkDao(object):
         protocol_checksum: int = self._get_protocol_checksum()
         if ifile_protocol_checksum != protocol_checksum:
             raise IOError(
-                "The info file contains a checksum that does not correspond to the actual protocol file."
+                "The info file contains a checksum that does not correspond to the "
+                "actual protocol file."
             )
 
     def _parse_info_file(self) -> Tuple[int, dt.datetime, int]:
@@ -262,16 +258,6 @@ class WorkDao(object):
             info_file_split: List[str] = info_file_content.split("/")
             info_file_match = re.fullmatch(INFO_FILE_PATTERN, info_file_content)
 
-        # <MIGRATE-info-file> functionality. To be removed in v0.95!
-        if re.fullmatch(
-            r"work-time-protocol/protocol-v(\d{1,2})/last-edit:(\d{4}-\d{2}-\d{2}\s\d{2}:\d{2})/hash:([^\n]+)\n",
-            info_file_content,
-        ):
-            raise IOError(
-                "Old info file version detected. "
-                "Please run `work rehash` once to update, then try again."
-            )
-
         if not len(info_file_split) == 4 or not info_file_match:
             raise IOError(f"Invalid info file at {self.info_file.resolve()}")
 
@@ -351,3 +337,51 @@ class WorkDao(object):
 
         # Nested creation (creates all parent directories, too)
         self.records_directory.mkdir(parents=True, exist_ok=True)
+
+    ### Low level classes ###
+
+    class RunFileDao:
+        """ DAO for the run file, useful for caching. """
+
+        def __init__(self, file: pathlib.Path) -> None:
+            self.file: pathlib.Path = file
+            self.cache: Optional[dt.datetime] = None
+
+        def exists(self) -> bool:
+            """ Direct interface to file.exists() """
+            return self.file.exists()
+
+        def resolve(self) -> pathlib.Path:
+            """ Direct interface to file.resolve() """
+            return self.file.resolve()
+
+        def unlink(self) -> None:
+            """ Direct interface to file.unlink() """
+            self.file.unlink()
+            self.cache = None
+
+        def write(self, start_time: dt.datetime) -> None:
+            """ Wrapper for file.open() and handler.write() """
+            with self.file.open(mode="w") as run_file_h:
+                run_file_h.write(start_time.strftime(DATETIME_FORMAT))
+            self.cache = start_time
+
+        def read(self) -> dt.datetime:
+            """
+            Wrapper for file.open(), handler.read(), and strptime(content).
+
+            Utilizes cached file content if available.
+            """
+            if self.cache is not None:
+                return self.cache
+
+            with self.file.open(mode="r") as run_file_h:
+                content: str = run_file_h.read()
+
+            try:
+                return dt.datetime.strptime(content, DATETIME_FORMAT)
+            except ValueError as val_error:
+                raise IOError(
+                    "Invalid run file! Please delete or fix. "
+                    f"(Path: {self.resolve()})"
+                ) from val_error
